︠f9016072-3bad-4b92-8da1-fa5eca55598ei︠
%html
<p>This sheet aims at illustrating a <strong>subdivision process</strong> to draw <strong>splines</strong>. A smooth curve is generated from a set of control points, and the subdivision process is used as an efficient way to approximate and draw the curve. It consists in recursively refining the set of control points, such that the polygonal line joining them tends to the spline.</p>
<p>This works is concerned about the way the subdivision scheme is defined, and illustrates an automatic process to generate one using an L-System.</p>
<p>&nbsp;</p>
<p><span style="font-size: x-large;"><strong>Code</strong></span></p>
<p><span style="font-size: x-large;"><span style="font-size: medium;">The following code intends to build a subdivision scheme from an input L-System describing the different intervals, and the way they are subdivided. It works as follows:</span></span></p>
<ul>
<li><span style="font-size: x-large;"><span style="font-size: medium;">From the provided L-System, a set of compatible lengths is computed for all the intervals present in the L-system (this length is the distance between two knots separated by the interval in parametric space).<br /></span></span></li>
<li><span style="font-size: x-large;"><span style="font-size: medium;">All the pole configuration are built. A pole configuration corresponds to a sequence of intervals, whose length depends on the desired continuity of the corresponding spline curve.</span></span></li>
<li><span style="font-size: x-large;"><span style="font-size: medium;">For each pole configuration, its sequence of intervals is subdivided, to derive all the new poles resulting in the subdivision of this pole.</span></span></li>
<li><span style="font-size: x-large;"><span style="font-size: medium;">For each new pole derived from an old pole, its stencil with respect to the old pole is computed (i.e. the influence coefficient of the old pole position in the new pole position). This uses knot insertion theory</span></span></li>
<li><span style="font-size: x-large;"><span style="font-size: medium;">For each pole configuration its mask is built by collecting the new poles it influences with the associated influence coefficient<br /></span></span></li>
</ul>

︡354b3220-bc4d-4601-935b-26e80fb71692︡{"html": "<p>This sheet aims at illustrating a <strong>subdivision process</strong> to draw <strong>splines</strong>. A smooth curve is generated from a set of control points, and the subdivision process is used as an efficient way to approximate and draw the curve. It consists in recursively refining the set of control points, such that the polygonal line joining them tends to the spline.</p>\n<p>This works is concerned about the way the subdivision scheme is defined, and illustrates an automatic process to generate one using an L-System.</p>\n<p>&nbsp;</p>\n<p><span style=\"font-size: x-large;\"><strong>Code</strong></span></p>\n<p><span style=\"font-size: x-large;\"><span style=\"font-size: medium;\">The following code intends to build a subdivision scheme from an input L-System describing the different intervals, and the way they are subdivided. It works as follows:</span></span></p>\n<ul>\n<li><span style=\"font-size: x-large;\"><span style=\"font-size: medium;\">From the provided L-System, a set of compatible lengths is computed for all the intervals present in the L-system (this length is the distance between two knots separated by the interval in parametric space).<br /></span></span></li>\n<li><span style=\"font-size: x-large;\"><span style=\"font-size: medium;\">All the pole configuration are built. A pole configuration corresponds to a sequence of intervals, whose length depends on the desired continuity of the corresponding spline curve.</span></span></li>\n<li><span style=\"font-size: x-large;\"><span style=\"font-size: medium;\">For each pole configuration, its sequence of intervals is subdivided, to derive all the new poles resulting in the subdivision of this pole.</span></span></li>\n<li><span style=\"font-size: x-large;\"><span style=\"font-size: medium;\">For each new pole derived from an old pole, its stencil with respect to the old pole is computed (i.e. the influence coefficient of the old pole position in the new pole position). This uses knot insertion theory</span></span></li>\n<li><span style=\"font-size: x-large;\"><span style=\"font-size: medium;\">For each pole configuration its mask is built by collecting the new poles it influences with the associated influence coefficient<br /></span></span></li>\n</ul>"}︡
︠7400f758-b215-4300-8f04-129a2ba3c333i︠
%hide

from numpy import array

# Subdivide a knot sequence
#
# Inputs:
# rules    : the subdivision rules
# intervals: the interval configuration of the knot sequence
#
# Outputs:
# newKnotConfig : the new knot sequence
# prevKnotConfig: the position of the old knots in the new knot sequence
# newIntervals  : the new interval configuration
#
# Example:
# rules    : S -> L | L -> SL
# intervals: SLL
#
# newKnotSequence : [0,1,None,2,None,3] (None is a new knot)
# prevKnotSequence: [0,1,3,5]
# intervals       : LSLSL

def subdivideConfig(rules,intervals):
    newKnotConfig = []
    newIntervals = []
    prevKnotConfig = []
    prevPos = 0
    currentKnot = 0
    for interval in intervals:
        # insert the orgininal knot starting the interval
        newKnotConfig.append(currentKnot)
        # insert its position within the new knot sequence
        prevKnotConfig.append(prevPos)
        # subdivide the interval
        subdiv = rules[interval]
        # amount of knots inserted
        newPoints = len(subdiv)-1
        #compute the position of the next original knot
        prevPos+=newPoints+1
        #extend the new interval configuration
        newIntervals.extend(subdiv)
        # for each inserted knot, add a None in the new knot sequence
        for dummy in range(newPoints):
            newKnotConfig.append(None)
        currentKnot+=1
    # add the last original knot in both knot sequences
    newKnotConfig.append(currentKnot)
    prevKnotConfig.append(prevPos)
    
    return (newKnotConfig,prevKnotConfig,newIntervals)

# Provides the parametric position of the knots corresponding to a given interval sequence
#
# Inputs:
# dicoLen  : the length of each interval of the grammar
# intervals: an interval sequence
#
# Ouputs:
# offsetList: the positions of the knots
#
# Example:
# dicoLen  : A -> 1 | B -> 2
# intervals: AABAB
#
# offsetList: [0,1,2,4,5,7]

def getOffsetList(dicoLen,intervals):
    current=0
    offsetList=[0]
    for interval in intervals:
        # for each interval add its length
        current+=dicoLen[interval]
        offsetList.append(current)
    return offsetList

# From the knot sequence corresponding to the decomposition of a coarse pole, computes the
# knot sequences corresponding to the new inserted poles, and the necessary information to 
# compute the influence of the coarse pole in the position of the new poles using knot insertion
#
# Inputs:
# knotList    : the knot sequence corresponding to the subdivided knot sequence of a coarse pole (provided by subdivideConfig)
# intervelList: the intervals between the knots of the knotList aforementionned
# size        : the size of the knot sequence corresponding to a pole (related to the continuity of the spline)
#
# Outputs:
# poles: the poles derived from the coarse subdivided pole (min,max,newknots,config)
#   -> min     : the index of the original knot directly before the first knot of the sequence +1 (in the coarse knot list)
#   -> max     : the index of the original knot directly after the last knot of the sequence -1 (in the coarse knot list)
#   -> newKnots: the indices of the new knots of the sequence of the pole (in fine knot list)
#   -> config  : the interval configuration of the pole
         
def getPoles(knotList,intervalList,size):
    # copy of the interval list
    intervals = intervalList[:]
    # to handle the termination
    intervals.append(None)
    # the sliding window corresponding to the config of the considered pole
    configQueue = [None]
    # the poles found so far
    poles = []
    # the new knot indices corresponding to the window
    newKnots = []
    # the knots corresponding to the window
    queue = []
    # the min of the pole corresponding to the window
    min = 0
    # the max of the pole corresponding to the window
    max = -1
    for (pos,knot) in enumerate(knotList):
        # we consider the knots in knotList one by one with their index
        if knot!=None:
            # the knot is an old one, increase the counter of the next old knot
            max = knot
        else:
            # the knot is a new one, add its position to the knew knot list
            newKnots.append(pos)
        # in all cases add the knot to the knot sequence of the window
        queue.append(knot)
        # add the next interval to the interval configuration of the window
        configQueue.append(intervals.pop(0))
        if len(queue)==size:
            # the size of the window corresponds to a pole, add it
            poles.append((min,max,newKnots[:],tuple(configQueue)))
            # sliding the window:
            # pop the first knot in the queue
            rem = queue[0]
            queue = queue[1:]
            if rem!=None:
                # the popped knot is an old one, and is the one right before the remaining knot sequence
                min = rem+1
            else:
                # the popped knot is an new one, remove it from the new knots of the window
                newKnots = newKnots[1:]
            #pop the first interval in the configuration
            configQueue.pop(0)
    return poles[1:-1]

# Computes the influence of a coarse pole in the position of a fine one, using knot insertion
#
# Inputs:
# offsetList: the shifts of the new knots relatively to the first knot in the sequence of the coarse pole
# size      : the size of the knot sequence of a pole (related to the continuity of the spline)
# prevPos   : the positions of the coarse knots within the refined knot sequence
# pole      : the considered pole (returned in the format of getPoles)
#
# Outputs:
# (coeff,before,after,config)
# coeff : the influence of the coarse pole in the new one
# before: the number of coarse poles before the considered one required to compute the new pole (negative)
# after : the number of poles after the considered one required to conpute the new pole
# config: the configuration of the new pole (copied from the input)

def stencil(offsetList,size,prevPos,pole):
    # recover the information of the pole
    min,max,remaining,config = pole
    if len(remaining)==0:
        #the pole is an original one since its knot sequence only contains coarse knots
        if min==1:
            #the original pole is that with respect to which the stencil is computed
            return (1,0,0,config)
        else:
            # the pole is not the desired one, min-1 is its offset from the considered one
            return (0,min-1,min-1,config)

    # the pole is computed using two poles (knot insertion).
    # These two poles are derived from the knot sequence of the pole:
    # -> a new knot is removed from the knot sequence
    # -> the knot sequence of the first pole is obtained by adding the old knot given by min-1 to the front
    # -> the knot sequence of the second pole is obtained by adding the old knot given by max+1 to the back
    prevPole = (min-1,max,remaining[1:],None)
    nextPole = (min,max+1,remaining[1:],None)
    #recursively compute the influence of the considered coarse pole in the two poles used in the knot insertion
    prevCoeff = stencil(offsetList,size,prevPos,prevPole)
    nextCoeff = stencil(offsetList,size,prevPos,nextPole)

    if min<1 or max>size:
        # it is directly possible to detect that the pole is not influenced by the considered coarse pole
        coeff=0
    else:
        #compute the coefficients of the knot insertion to form the pole using the knot insertion
        startPos = offsetList[prevPos[min-1]]
        endPos = offsetList[prevPos[max+1]]
        pos = offsetList[remaining[0]]
        coeff = ((endPos-pos)*prevCoeff[0]+(pos-startPos)*nextCoeff[0])/(endPos-startPos)
    return (coeff,prevCoeff[1],nextCoeff[2],config)

# Checks if all the elements of a list have the same sign

def allSameSign(l):
    sign = l[0]
    check = [x*sign for x in l]
    for x in check:
      if x<0: return False
    return True

# Given a subdivision mask and a pole sequence, subdivides the pole sequence according to the mask
#
# Inputs:
# subdivMask: the subdivision mask
# poles     : the pole sequence to be subdivided
#
# Outputs:
# newPoints: the new pole sequence

def applySubdivision(subdivMask,poles):
    # the new computed poles
    newPoints = []
    # the poles being computed
    onGoingPoints = []
    prevCount = 0
    for (coord,config) in poles:
        # for each old pole, recover its subdivision mask
        mask = subdivMask[config]
        # get the number of poles being computed
        overlap = len(onGoingPoints)
        ndrop = 0
        for count,(coeff,min,max,config) in enumerate(mask):
            # each pole in the decompositionof the old pole is considered
            if prevCount+min<0:
                # the pole cannot be computed, lacking old poles (starting end of the curve)
                ndrop +=1
                continue
            if count < overlap+ndrop:
                # the pole computation is on the go, started from a previous old pole
                # pop the current state from ongoing poles
                (pos,rest) = onGoingPoints.pop(0)
                # add the contribution of the considered old pole
                pos = pos+coord*coeff
                if rest==1:
                    # the computation of this pole is completed, add it to the resulting poles
                    newPoints.append((pos,config))
                else:
                    # the computation is not finished yet, put it back in the list
                    onGoingPoints.append((pos,rest-1))
            else:
                # the new pole computation was not previously started
                pos = coeff*coord
                if max>0:
                    # the new pole needs the following old poles to be computed
                    onGoingPoints.append((coeff*coord,max))
                else:
                    # the new pole is the decomposed old pole, coeff is 1
                    newPoints.append((coord,config))
        prevCount+=1
    return newPoints
    
    
# Builds the subdivision mask of a pole configuration
#
# Inputs:
# dicoRules: subdivision rules
# dicoLen  : lengths of all the interval types present in the subdivision rules
# intervals: the configuration of the considered pole
#
# Outputs:
# [(coeff,before,after,config)...]
# -> coeff : the influence of the pole in the construction of a new one
# -> before: the amount of necessary poles before the considered one to build this new one (negative)
# -> after : the amount of necessary poles after the considered one to build this new one (positive)
# -> config: the configuration of the new pole

def buildMask(dicoRules,dicoLen,intervals):
    size = len(intervals)-1
    # sibdivide the knot sequence of the input pole to create the new knot sequence
    config,prevPos,newIntervals = subdivideConfig(dicoRules,intervals)
    # extract all the new poles from the new knot sequence
    newPoles = getPoles(config,newIntervals,size)
    # compute the repartition of the knots using the interval lengths
    offsetList = getOffsetList(dicoLen,newIntervals)
    # for each new pole, compute the influence of the considered old one
    return [pole for pole in map(lambda x : stencil(offsetList,size,prevPos,x),newPoles) if pole[0]]

# Computes the lengths of the intervals from the subdivision rules L-system
#
# Inputs:
# dicoRules: the subdivision rules L-system
#
# Outputs:
# dicoLen: a list of all the possible sets of lengths for the intervals present in the L-system

def computeLen(dicoRules):
    # convert from index to interval and vice versa
    intervalToIndex = {}
    indexToInterval =  []
    for index,interval in enumerate(dicoRules.keys()):
        indexToInterval.append(interval)
        intervalToIndex[interval] = index
        
    # create the matrix corresponding to the L-System
    dicoMat = {}
    for index,decomposition in enumerate(dicoRules.values()):
        for interval in decomposition:
            pos = (index,intervalToIndex[interval])
            if dicoMat.has_key(pos):
                dicoMat[pos]+=1
            else:
                dicoMat[pos] =1
           
    M = matrix(QQ,dicoMat)
    
    # extract the eigenvalues and corresponding eigenvectors
    eigenStruct = M.eigenvectors_right()
    
    # get all the matching couples with eigenvalue greater than 1 and an eigenvector with components of identical sign
    candidates = filter(lambda (val,vect,mul): val>1 and allSameSign(vect),eigenStruct)
    
    #extract the lengths of all the intervals
    candidates.sort()
    dicoLen = []
    for (ratio,lengths,mul) in candidates:
        dic = {}
        for (index,interval) in enumerate(indexToInterval):
            dic[interval]=lengths[0][index]
        dicoLen.append(dic)
    return dicoLen

# Create all the possible interval sequence configurations from the L-system
#
# Inputs:
# size     : the size of the configurations
# dicoRules: the subdivision L-system
#
# Outputs:
# [configuration,...]
# -> configuration: [interval,...]
#
# Example:
# size = 3
# dicoRules = {S->L | L->SL}
# 
# result: [[S,S,S],[S,S,L],[S,L,S],[S,L,L],[L,S,S],[L,S,L],[L,L,S],[L,L,L]]

def generateMaskConfigurations(size,dicoRules,prevConfigs = [[]]):
    if size == 0:
        # if the size is 0 no additional pass is done, return the configurations built so far
        return prevConfigs
    else:
        newConfigs = []
        # for each symbol in the L-system, create a new set of configurations,
        # by appending all the existing smaller configurations to it
        for elem in dicoRules.keys():
            newConfigs.extend([[elem]+prevConfig for prevConfig in prevConfigs])
        return generateMaskConfigurations(size-1,dicoRules,newConfigs)
 
# Builds a subdivision scheme
#
# Inputs:
# rules     : the L-System describing the subdivision rules
# continuity: the desired continuity
#
# Outputs:
# masks: for each possible interval sequence, associates its subdivision mask

def buildSubdivision(rules,continuity):
    # compute the possible lengths for the intervals of the L-system
    dicoLens = computeLen(rules)
    if len(dicoLens)<1:
        print "The set of rules is not suitable for subdivision"
        return
    if len(dicoLens)>1:
        print "Warning: more than one possible subdivision scheme, using smallest ratio"
    dicoLen = dicoLens[0]
    # the size of the interval configurations of the poles is related to the desired continuity
    size = continuity+2
    # generate all the possible interval sequence configurations
    configurations = generateMaskConfigurations(size,rules)
    # for each interval sequence configuration, build the corresponding subdivision mask
    masks = dict([(tuple(conf),buildMask(rules,dicoLen,conf)) for conf in configurations])
    return masks

# Plot the polygonal line defined by a list of poles
#
# Inputs:
# poles : the list of poles of the form (position,configuration)
# colors: (optional) a dictionnary associating a color for each interval
#
# Outputs:
# g: the graphicall representation

def plotPoles(poles,colors=None):
    previousPole = None
    previousInterval = None
    g = Graphics()
    for coord,config in poles:
	if previousPole!= None:
	  if colors:
	    color = colors[previousInterval]
	  else:
	    color='red'
	  g += line([[real(coord[0]),real(coord[1])] , [real(previousPole[0]),real(previousPole[1])]],rgbcolor=color)
	previousPole = coord
	previousInterval = config[0]
    g+= points([[real(coord[0]),real(coord[1])] for (coord,config) in poles],rgbcolor='black')
    return g

# Generate a random initial knot configuration
#
# Inputs:
# rules     : the subdivision L-System
# size      : the number of poles
# continuity: the continuity of the scheme (to determine the length of the configuration of the poles)
#
# Outputs:
# points: a list of random poles with coherent configurations

def generateExample(rules,size,continuity):
    config = []
    intervals = rules.keys()
    nint = len(intervals)
    points = []
    getInterval = lambda : intervals[floor(random()*nint)]
    for i in range(continuity+2):
        config.append(getInterval())
    for i in range(size):
        x,y = (random(),random())
        points.append((array([x,y]),tuple(config[:])))
        config.pop(0)
        config.append(getInterval())
    return points
︡02b33a06-69cd-4f57-ae49-a9d4c03184a8︡︡
︠bdf9a65e-3959-40ff-b8be-7502012aeb67i︠
%html
<p><span style="font-size: x-large;"><strong>Examples</strong></span></p>
<p><span style="font-size: x-large;"><span style="font-size: medium;">We here build two examples of subdivision schemes:</span></span></p>
<ul>
<li><span style="font-size: x-large;"><span style="font-size: medium;">The bisecting-trisecting scheme $B \rightarrow TT, T \rightarrow BBB$</span></span></li>
</ul>
<ul>
<li><span style="font-size: x-large;"><span style="font-size: medium;">The Fibonacci scheme </span></span><span style="font-size: x-large;"><span style="font-size: medium;">$S \rightarrow L, L \rightarrow SL$</span></span></li>
</ul>

︡29f2a618-4322-4bd6-942c-6ca7d4986f8c︡{"html": "<p><span style=\"font-size: x-large;\"><strong>Examples</strong></span></p>\n<p><span style=\"font-size: x-large;\"><span style=\"font-size: medium;\">We here build two examples of subdivision schemes:</span></span></p>\n<ul>\n<li><span style=\"font-size: x-large;\"><span style=\"font-size: medium;\">The bisecting-trisecting scheme $B \\rightarrow TT, T \\rightarrow BBB$</span></span></li>\n</ul>\n<ul>\n<li><span style=\"font-size: x-large;\"><span style=\"font-size: medium;\">The Fibonacci scheme </span></span><span style=\"font-size: x-large;\"><span style=\"font-size: medium;\">$S \\rightarrow L, L \\rightarrow SL$</span></span></li>\n</ul>"}︡
︠f2a9a851-6fa2-4ea4-a5f7-8708b374e5a5︠
#bisecting and trisecting scheme rules
bitriRules = {'T':['B','B','B'],'B':['T','T']}
#generate subdivision mask
bitriMask = buildSubdivision(bitriRules,2)
#color of each interval for the plots
bitriColors = {'B':(0.6,0.6,1),'T':(1,0.6,0.6)}

#fibonacci scheme rules
fiboRules = {'S':['L'],'L':['S','L']}
#generate subdivision mask
fiboMask = buildSubdivision(fiboRules,2)
#color of each interval for the plots
fiboColors = {'S':(0.6,0.6,1),'L':(1,0.6,0.6)}
︡7b0d4699-3bf4-45fe-80cc-76cc8cddc0a3︡︡
︠c63412dc-5bfa-4c0a-8db5-ddbf02d492cd︠
#random initialization of the control points
poles = generateExample(fiboRules,20,2)

#plot the initial control points
plotPoles(poles,fiboColors).show(axes=False,xmin=0,xmax=1,ymin=0,ymax=1,figsize=(3,3))
︡e5320a81-a077-4e8c-acf5-dc61c831081c︡{"html": "<font color='black'><img src='cell://sage0.png'></font>"}︡
︠378cb141-ff26-4984-ad2f-17af2246f964︠
mask = fiboMask
colors = fiboColors

subdivPoles  = applySubdivision(mask,poles)
subdivPoles1 = applySubdivision(mask,subdivPoles)
subdivPoles2 = applySubdivision(mask,subdivPoles1)
subdivPoles3 = applySubdivision(mask,subdivPoles2)
subdivPoles4 = applySubdivision(mask,subdivPoles3)
subdivPoles5 = applySubdivision(mask,subdivPoles4)
subdivPoles6 = applySubdivision(mask,subdivPoles5)
graphs = [plotPoles(poles,colors)]
graphs.extend([plotPoles(p,colors) for p in [subdivPoles,subdivPoles1,subdivPoles2,subdivPoles3,subdivPoles4,subdivPoles5,subdivPoles6]])
graphics_array(graphs,2,3).show(axes=False,xmin=0,xmax=1,ymin=0,ymax=1,figsize=(12,8))
graphics_array(graphs,2,3).save(filename='phiSpline.pdf',axes=False,xmin=0,xmax=1,ymin=0,ymax=1,figsize=(12,8))
︡1030914f-def0-4930-bfe9-05f98dd2087e︡︡
︠9db3bf80-3e30-4d61-b845-77e1ebefc547i︠
%html
<p>The complete subdivision mask contains all the possible pole configurations. However, after several subdivision steps, only a few configurations are usefull. These can be found as part of the strongly connected components of the directed graph derived from the subdivision mask. Here is an example with the case of the Fibonacci mask computed above. The after a few subdivision steps, the blue labelled configurations are the only ones used for the subdivided poles.</p>

︡52723236-829b-456d-b962-9368cdf8a5a2︡{"html": "<p>The complete subdivision mask contains all the possible pole configurations. However, after several subdivision steps, only a few configurations are usefull. These can be found as part of the strongly connected components of the directed graph derived from the subdivision mask. Here is an example with the case of the Fibonacci mask computed above. The after a few subdivision steps, the blue labelled configurations are the only ones used for the subdivided poles.</p>"}︡
︠ccca351c-c760-450e-b838-cd4eec08ea57︠
#create a string from an interval sequence
def label(config):
    return(''.join(config))

#turn the subdivision mask into a directed graph
def graphFromMask(mask):
    d = {}
    for source,ends in mask.items():
        d[label(source)] = map(lambda x: label(x[3]),ends)
    return(DiGraph(d))
    
#Creates colors to mark the strongly connected components
ccColor = (0.6,0.6,1)
noColor = (1,0.6,0.6)

def colorComponents(graph):
    c = graph.strongly_connected_components()
    colors = {noColor:[],ccColor:[]}
    for component in c:
        if len(component)==1:
            colors[noColor].extend(component)
        else:
            colors[ccColor].extend(component)
    return colors

g = graphFromMask(fiboMask)
c = colorComponents(g)
p = g.plot(vertex_colors=c)
p.show(figsize=(10,10))
p.save(filename='maskComponents.pdf',figsize=[10,10])
︡57bdd993-03e4-483c-9b45-de8a661d883c︡{"html": "<font color='black'><img src='cell://sage0.png'></font>"}︡
︠63c6be69-253a-4ff1-a8b3-93b0c35a17a5i︠
%html
<p><strong><span style="font-size: x-large;">Interactive Application</span></strong></p>
<p>The following is an interactive application demonstrating the computation of a subdivision scheme from an input L-System. The L-System must be given in the form of a python dictionnary as done in the above code for the bitriRules and the fiboRules variables. It is currently not possible to use the interactive application in the published version, since it requires effectively running the present code. To do so, you must either download the sheet and run it in your local sage installation, or edit your own copy of the present document, which requires creating a (free) user account on this server.</p>

︡dd072316-3d6a-4f17-b422-446efc1f51dc︡{"html": "<p><strong><span style=\"font-size: x-large;\">Interactive Application</span></strong></p>\n<p>The following is an interactive application demonstrating the computation of a subdivision scheme from an input L-System. The L-System must be given in the form of a python dictionnary as done in the above code for the bitriRules and the fiboRules variables. It is currently not possible to use the interactive application in the published version, since it requires effectively running the present code. To do so, you must either download the sheet and run it in your local sage installation, or edit your own copy of the present document, which requires creating a (free) user account on this server.</p>"}︡
︠97958428-dec1-4c10-896d-6081fe5c58bei︠
%hide

app_masks = {}
app_poles = []
app_graphs = []
app_prevShow = ("",0,0)

@interact
def _(rules = {'L':['S','L'],'S':['L']}, continuity=(2,(0..10)),points_in_example=(15,(0..100)),steps=(3,(0..10))):
    global app_masks,app_poles,app_graphs,app_prevShow
    if rules:
        mask=None
        ruleKey = str(rules)
        if app_masks.has_key(ruleKey):
            continuities = app_masks[ruleKey]
            if len(continuities)>continuity:
                mask = continuities[continuity]
            else:
                diff = continuity-len(continuities)+1
                continuities.extend([None for i in range(diff)])
        else:
            app_masks[ruleKey] = [None for i in range(continuity+1)]
        if not mask:
            mask = buildSubdivision(rules,continuity)
        if (app_prevShow!=(ruleKey,continuity,points_in_example)):
            app_prevShow = (ruleKey,continuity,points_in_example)
            app_poles = [generateExample(rules,points_in_example,continuity)]
            app_graphs = [plotPoles(app_poles[0])]
        if len(app_poles)<=steps:
            for i in range(steps-len(app_poles)+1):
                app_poles.append(applySubdivision(mask,app_poles[-1]))
        if len(app_graphs)<=steps:
            app_graphs.extend([None for i in range(steps-len(app_graphs)+1)])
        graph = app_graphs[steps]
        if not graph:
            graph = plotPoles(app_poles[steps])
            app_graphs[steps] = graph
        show(graphics_array([app_graphs[0],app_graphs[steps]]),axes=False,xmin=0,xmax=1,ymin=0,ymax=1,figsize=(8,4))
    else:
        prevShow = (None,continuity,points_in_example)
︡0210c2b8-b8f7-42ef-9522-48aa4318b4b3︡{"html": "<!--notruncate--><div padding=6 id=\"div-interact-18\"> <table width=800px height=20px bgcolor=\"#c5c5c5\"\n                 cellpadding=15><tr><td bgcolor=\"#f9f9f9\" valign=top align=left><table><tr><td align=right><font color=\"black\">rules&nbsp;</font></td><td><input type=\"text\" value=\"{'S': ['L'], 'L': ['S', 'L']}\" size=80 onchange=\"interact(18, '_interact_.update(\\'18\\', \\'rules\\', 5, _interact_.standard_b64decode(\\''+encode64(this.value)+'\\'), globals()); _interact_.recompute(\\'18\\');')\"></input></td></tr>\n<tr><td align=right><font color=\"black\">continuity&nbsp;</font></td><td><table><tr><td>\n        <div id=\"slider-continuity-18\" style=\"margin:0px; margin-left: 1.0em; margin-right: 1.0em; width: 15.0em;\"></div>\n        </td><td><font color=\"black\" id=\"slider-continuity-18-lbl\"></font></td></tr></table><script>(function(){ var values = [\"0\",\"1\",\"2\",\"3\",\"4\",\"5\",\"6\",\"7\",\"8\",\"9\",\"10\"]; setTimeout(function() {\n    $('#slider-continuity-18').slider({\n        step: 1, min: 0, max: 10, value: 2,\n        change: function (e,ui) { var position = ui.value; if(values!=null) $('#slider-continuity-18-lbl').text(values[position]); interact(18, '_interact_.update(\\'18\\', \\'continuity\\', 6, _interact_.standard_b64decode(\\''+encode64(position)+'\\'), globals()); _interact_.recompute(\\'18\\');'); },\n        slide: function(e,ui) { if(values!=null) $('#slider-continuity-18-lbl').text(values[ui.value]); }\n    });\n    if(values != null) $('#slider-continuity-18-lbl').text(values[$('#slider-continuity-18').slider('value')]);\n    }, 1); })();</script></td></tr>\n<tr><td align=right><font color=\"black\">points_in_example&nbsp;</font></td><td><table><tr><td>\n        <div id=\"slider-points_in_example-18\" style=\"margin:0px; margin-left: 1.0em; margin-right: 1.0em; width: 15.0em;\"></div>\n        </td><td><font color=\"black\" id=\"slider-points_in_example-18-lbl\"></font></td></tr></table><script>(function(){ var values = [\"0\",\"1\",\"2\",\"3\",\"4\",\"5\",\"6\",\"7\",\"8\",\"9\",\"10\",\"11\",\"12\",\"13\",\"14\",\"15\",\"16\",\"17\",\"18\",\"19\",\"20\",\"21\",\"22\",\"23\",\"24\",\"25\",\"26\",\"27\",\"28\",\"29\",\"30\",\"31\",\"32\",\"33\",\"34\",\"35\",\"36\",\"37\",\"38\",\"39\",\"40\",\"41\",\"42\",\"43\",\"44\",\"45\",\"46\",\"47\",\"48\",\"49\",\"50\",\"51\",\"52\",\"53\",\"54\",\"55\",\"56\",\"57\",\"58\",\"59\",\"60\",\"61\",\"62\",\"63\",\"64\",\"65\",\"66\",\"67\",\"68\",\"69\",\"70\",\"71\",\"72\",\"73\",\"74\",\"75\",\"76\",\"77\",\"78\",\"79\",\"80\",\"81\",\"82\",\"83\",\"84\",\"85\",\"86\",\"87\",\"88\",\"89\",\"90\",\"91\",\"92\",\"93\",\"94\",\"95\",\"96\",\"97\",\"98\",\"99\",\"100\"]; setTimeout(function() {\n    $('#slider-points_in_example-18').slider({\n        step: 1, min: 0, max: 100, value: 15,\n        change: function (e,ui) { var position = ui.value; if(values!=null) $('#slider-points_in_example-18-lbl').text(values[position]); interact(18, '_interact_.update(\\'18\\', \\'points_in_example\\', 7, _interact_.standard_b64decode(\\''+encode64(position)+'\\'), globals()); _interact_.recompute(\\'18\\');'); },\n        slide: function(e,ui) { if(values!=null) $('#slider-points_in_example-18-lbl').text(values[ui.value]); }\n    });\n    if(values != null) $('#slider-points_in_example-18-lbl').text(values[$('#slider-points_in_example-18').slider('value')]);\n    }, 1); })();</script></td></tr>\n<tr><td align=right><font color=\"black\">steps&nbsp;</font></td><td><table><tr><td>\n        <div id=\"slider-steps-18\" style=\"margin:0px; margin-left: 1.0em; margin-right: 1.0em; width: 15.0em;\"></div>\n        </td><td><font color=\"black\" id=\"slider-steps-18-lbl\"></font></td></tr></table><script>(function(){ var values = [\"0\",\"1\",\"2\",\"3\",\"4\",\"5\",\"6\",\"7\",\"8\",\"9\",\"10\"]; setTimeout(function() {\n    $('#slider-steps-18').slider({\n        step: 1, min: 0, max: 10, value: 3,\n        change: function (e,ui) { var position = ui.value; if(values!=null) $('#slider-steps-18-lbl').text(values[position]); interact(18, '_interact_.update(\\'18\\', \\'steps\\', 8, _interact_.standard_b64decode(\\''+encode64(position)+'\\'), globals()); _interact_.recompute(\\'18\\');'); },\n        slide: function(e,ui) { if(values!=null) $('#slider-steps-18-lbl').text(values[ui.value]); }\n    });\n    if(values != null) $('#slider-steps-18-lbl').text(values[$('#slider-steps-18').slider('value')]);\n    }, 1); })();</script></td></tr>\n</table><div id=\"cell-interact-18\"><?__SAGE__START>\n        <table border=0 bgcolor=\"white\" width=100% height=100%>\n        <tr><td bgcolor=\"white\" align=left valign=top><pre><?__SAGE__TEXT></pre></td></tr>\n        <tr><td  align=left valign=top><?__SAGE__HTML></td></tr>\n        </table><?__SAGE__END></div></td>\n                 </tr></table></div>\n                 "}︡
